from django.core.validators import RegexValidator
from django.db import models

from wagtail.admin.edit_handlers import FieldPanel, MultiFieldPanel
from wagtail.images.edit_handlers import ImageChooserPanel

from taggit.managers import TaggableManager

phone_number_valitator = RegexValidator(
    regex=r'^\+?[\d\(\) ]{4,20}$',
    message="Veuillez saisir un numéro de téléphone valide",
)


class Organization(models.Model):
    """
    Une organisation et les informations de base qui lui sont associées
    """

    name = models.CharField("Nom", max_length=128, unique=True)
    logo = models.ForeignKey(
        'wagtailimages.Image',
        related_name='+',
        verbose_name="Logo",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )
    activity = TaggableManager(
        verbose_name="Secteur(s) d'activité",
        help_text="En cas de secteurs multiples, séparez les par une virgule",
    )
    city = models.CharField("Ville", max_length=64)

    # Determines if the organization is displayed in the inhabitants phonebook
    inhabitant = models.BooleanField(
        "Habitant du Toit Commun",
        help_text=(
            "Cocher ici si la structure doit apparaitre sur l'annuaire "
            "des habitants du Toit Commun."
        ),
        default=False,
    )

    description = models.TextField(
        "Description",
        help_text=(
            "Une description succinte de l'organisation et de son activité."
        ),
    )
    phone = models.CharField(
        "Numéro de téléphone",
        max_length=21,
        validators=[phone_number_valitator],
    )
    email = models.EmailField("Adresse mail")
    address = models.CharField(
        "adresse",
        max_length=255,
        blank=True,
        help_text="L'adresse postale affichée sur une ligne.",
    )

    # Attributs Wagtail
    panels = [
        MultiFieldPanel(
            [
                FieldPanel('name'),
                ImageChooserPanel('logo'),
                FieldPanel('activity'),
                FieldPanel('description'),
                FieldPanel('city'),
                FieldPanel('inhabitant'),
            ],
            heading="Informations générales",
        ),
        MultiFieldPanel(
            [
                FieldPanel('phone'),
                FieldPanel('email'),
                FieldPanel('address'),
            ],
            heading="Informations de contact",
        ),
    ]

    class Meta:
        verbose_name = "Organisation"
        verbose_name_plural = "Organisations"
        ordering = ('name',)

    def __str__(self):
        return self.name

    def display_activities(self):
        return ", ".join([tag.name for tag in self.activity.all()])
